# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - Wiki Security Interface

    This implements the basic interface for user permissions and
    system policy. If you want to define your own policy, inherit
    from the base class 'Permissions', so that when new permissions
    are defined, you get the defaults.

    Then assign your new class to "SecurityPolicy" in wikiconfig;
    and I mean the class, not an instance of it!

    @copyright: 2000-2004 by Jrgen Hermann <jh@web.de>
    @license: GNU GPL, see COPYING for details.
"""

#############################################################################
### Basic Permissions Interface -- most features enabled by default
#############################################################################


class Permissions:
    """ Basic interface for user permissions and system policy.

    Note that you still need to allow some of the related actions, this
    just controls their behavior, not their activation.

    When sub classing this class, you must extend the class methods, not
    replace them, or you might break the acl in the wiki. Correct sub
    classing look like this:

    def read(self, pagename):
        # Your special security rule
        if somehting:
            return false

        # Do not return True or you break acl!
        # This call will use the default acl rules
        return Permissions.read(pagename)
    """

    def __init__(self, user):
        """ Calculate the permissons `user` has.
        """
        from MoinMoin.Page import Page
        self.Page = Page
        self.name = user.name
        self.user = user
        self.request = user._request

    def save(self, editor, newtext, rev, **kw):
        """ Check whether user may save a page.

        Default implementation ignore newtext, rev and kw, and just try
        to get permission for write.

        @param editor: PageEditor instance.
        @param newtext: new page text, you can enable of disable saving according
            to the content of the text, e.g. prevent link spam.
        @param rev: new revision number? XXX
        @param kw: XXX
        @rtype: bool
        @return: True if you can save or False
        """
        return self.write(editor.page_name)

    def __getattr__(self, attr):
        """ Shortcut to export getPermission function for all known acl rights
        
        if attr is one of the rights in acl_rights_valid, then return a
        checking function for it. Else use normal getattr().

        @param attr: one of acl known rights as defined in acl_rights_valid
        @rtype: function
        @return: checking function for that right, accepting a pagename
        """
        if attr in self.request.cfg.acl_rights_valid:
            self._right = attr
            return self._getPermission
        else:
            raise AttributeError, attr
        
    def _getPermission(self, pagename):
        """ Get permission by transversing page hierarchy
 
        If cfg.acl_hierarchic, we check each page in the hierarchy. If all
        pages agree, we have permission. If one page in the hierarchy
        does not agree, we don't. Otherwise, only pagename is checked.

        This method should not be called by users, use __getattr__
        instead. If you want to get permission for delete, try:
        instance.delete(pagename).

        @param pagename: pagename to get permission from
        @rtype: bool
        @return: True if you have permission or False
        """
        from MoinMoin.Page import Page

        # Use page hierarchy or only pagename, according to wiki config.
        if self.request.cfg.acl_hierarchic:
            pages = pagename.split('/')
            #open('/tmp/acl.log','a').write('acl_hierarchic enabled\n')
        else:
            #open('/tmp/acl.log','a').write('acl_hierarchic disabled\n')
            pages = [pagename]

        # disable logging seems not bugfree
        #open('/tmp/acl.log','a').write('may(%s): pages %s\n'%(self._right, pages,))
        # check before
        allowed = self.request.cfg.acl_before.may(self.request, self.user.name, self._right)
        if allowed is not None:
            #open('/tmp/acl.log','a').write('may: before returns %s\n'%(allowed,))
            return allowed
        
        # Get permission
        some_acl = False
        for i in range(len(pages), 0, -1):
            # Create the next pagename in the hierarchy
            # starting with the leave, going to the root
            name = '/'.join(pages[:i])
            # Get page acl and ask for permission
            acl = Page(self.request, name).getACL(self.request)
            #open('/tmp/acl.log','a').write('may: checking acl of %s (%s)\n'%(name, `acl.acl`))
            if acl.acl:
                some_acl = True
                allowed = acl.may(self.request, self.user.name, self._right)
                if allowed is not None:
                    #open('/tmp/acl.log','a').write('may: acl of %s (%s) returns %s\n'%(name, `acl.acl`, allowed,))
                    return allowed
        if not some_acl:
            allowed = self.request.cfg.acl_default.may(self.request, self.user.name, self._right)
            if allowed is not None:
                #open('/tmp/acl.log','a').write('may: default returns %s\n'%(allowed,))
                return allowed

        # check after
        allowed = self.request.cfg.acl_after.may(self.request, self.user.name, self._right)
        if allowed is not None:
            #open('/tmp/acl.log','a').write('may: after returns %s\n'%(allowed,))
            return allowed

        # sane default permission, if you want True,
        # just put 'All:read' (or more) into your acl_rights_after
        return False


# make an alias for the default policy
Default = Permissions

